module Main where

import Control.Monad
import Control.Concurrent (threadDelay)
import Data.IORef

import System.Random

import qualified Graphics.UI.Threepenny as UI
import Graphics.UI.Threepenny.Core

import Minesweeper

main :: IO ()
main = do
    startGUI defaultConfig {jsStatic = Just "view"} setup

setup :: Window -> UI ()
setup w = void $ do
    return w # set title "Minesweeper"

    UI.addStyleSheet w "bootstrap.min.css"
    UI.addStyleSheet w "minesweeper.css"

    rng <- liftIO newStdGen
    
    iob <- liftIO $ newIORef (createBoard 30 0.15 rng :: Board)
    b <- liftIO $ readIORef iob

    getBody w #+ [
        UI.div #. "container" #+ [
            UI.h1 #+ [string "Minesweeper"],
            UI.h4 #+ [string "Jack Harley jharley@tcd.ie"],
            mkElement "table" #+ rows iob 0
        ]]

rows :: IORef Board -> Int -> [UI Element]
rows iob r = do
    b <- liftIO $ readIORef iob

    rows' iob b r

    where
        rows' iob b r | r < size b = (mkElement "tr" #+ cells iob b r 0) : rows' iob b (r+1)
                      | otherwise = []

cells :: IORef Board -> Board -> Int -> Int -> [UI Element]
cells iob b r c = do
    cells' iob b r c

    where
        cells' iob b r c | c < size b = cell iob b (r,c) : cells' iob b r (c+1)
                         | otherwise = []


cell :: IORef Board -> Board -> Square -> UI Element
cell iob b (r,c) = do
    cell <- mkElement "td" #. (squareBgColour b (r,c) ++ " " ++ squareTextColour b (r,c)) 
                           #+ [string $ squareAscii b (r,c)]

    on UI.click cell $ \_ -> do
        liftIO $ modifyIORef' iob $ \oldB -> uncover oldB (r,c)

    return cell